//! @file usb_standard_request.c,v
//!
//! Copyright (c) 2004 Atmel.
//!
//! Use of this program is subject to Atmel's End User License Agreement.
//! Please read file license.txt for copyright notice.
//!
//! @brief Process USB device enumeration requests.
//!
//! This file contains the USB endpoint 0 management routines corresponding to
//! the standard enumeration process (refer to chapter 9 of the USB
//! specification.
//! This file calls routines of the usb_specific_request.c file for non-standard
//! request management.
//! The enumeration parameters (descriptor tables) are contained in the
//! usb_descriptors.c file.
//!
//! @version 1.5 at90usb128-demo-template-1_0_5 $Id: usb_device_request.c,v 1.5 2008-03-28 10:11:31 tyridal Exp $
//!
//! @todo
//! @bug

//_____ I N C L U D E S ____________________________________________________


#include <ioavr.h>
#include "conf_usb.h"
#include "usb_avr_registers.h"
#include "usb.h"
#include "descriptors.h"
#include "usb_device_request.h"
//#include "usb_descriptors.h"
//#include "modules\usb\device_chap9\usb_standard_request.h"
//#include "usb_specific_request.h"

#if ((USB_DEVICE_FEATURE == ENABLED)&& (USB_HOST_FEATURE == DISABLE))


//_____ M A C R O S ________________________________________________________


//_____ D E F I N I T I O N ________________________________________________

//_____ P R I V A T E   D E C L A R A T I O N ______________________________

static  void    usb_get_descriptor(U8   bmRequestType);
static  void    usb_set_address(      void);
static  void    usb_set_configuration(void);
#ifndef LEAN_USB
static  void    usb_clear_feature(    U8 bmRequestType);
static  void    usb_set_feature(      U8 bmRequestType);
static  void    usb_get_status(       U8 bmRequestType);
static  void    usb_get_configuration(void);
#endif
static  void    usb_get_interface (void);
static  void    usb_set_interface (void);





//_____ D E C L A R A T I O N ______________________________________________

#ifndef LEAN_USB
static  U8   endpoint_status[NB_ENDPOINTS];
#endif

//        U16  wInterface;

        U8   usb_configuration_nb;
extern  bit     usb_connected;



//! usb_process_request.
//!
//! @brief This function reads the SETUP request sent to the default control endpoint
//! and calls the appropriate function. When exiting of the usb_read_request
//! function, the device is ready to manage the next request.
void usb_process_request(void)
{
   U8 bmRequestType = Usb_read_byte();
   U8 bmRequest     = Usb_read_byte();

   switch (bmRequest)
   {
    case GET_DESCRIPTOR:
         usb_get_descriptor(bmRequestType); 
         break;

    case SET_ADDRESS:
         if (0x00 == bmRequestType) { usb_set_address(); }
         else                       
         { 
             goto usb_process_request_unsupported;
         }
         break;

    case SET_CONFIGURATION:
         if (0x00 == bmRequestType) { usb_set_configuration(); }
         else                       
         { 
             goto usb_process_request_unsupported;
         }
         break;
#ifndef LEAN_USB
    case GET_CONFIGURATION:
         if (0x80 == bmRequestType) { usb_get_configuration(); }
         else                       
         { 
             goto usb_process_request_unsupported;
         }
         break;

    case CLEAR_FEATURE:
         if (ENDPOINT_TYPE == bmRequestType) { usb_clear_feature(bmRequestType); }
         else if (!usb_user_process_request(bmRequestType,bmRequest))
         {
             goto usb_process_request_unsupported;
         }
         break;

    case SET_FEATURE:
         if (0x02 >= bmRequestType) { usb_set_feature(bmRequestType); }
         else                       { 
             goto usb_process_request_unsupported;
         }
         break;

    case GET_STATUS:
         if ((0x7F < bmRequestType) & (0x82 >= bmRequestType))
                                    { usb_get_status(bmRequestType); }
         else if (! usb_user_process_request(bmRequestType,bmRequest))
         {
             goto usb_process_request_unsupported;
         }
         break;
#endif
    case GET_INTERFACE:
         if (bmRequestType == 0x81) { usb_get_interface(); }
         else if (! usb_user_process_request(bmRequestType,bmRequest))
         {
             goto usb_process_request_unsupported;
         }
         break;


    case SET_INTERFACE:
         if (bmRequestType == 0x01) {usb_set_interface();}
         else if (! usb_user_process_request(bmRequestType,bmRequest))
         {
             goto usb_process_request_unsupported;
         }
         break;

    case SET_DESCRIPTOR:
    case SYNCH_FRAME:
    default: //!< un-supported request => call to user read request
        if (! usb_user_process_request(bmRequestType, bmRequest))
        {    
            goto usb_process_request_unsupported;
  
        }
   }

  return;

usb_process_request_unsupported:
    Usb_enable_stall_handshake();
    Usb_ack_receive_setup();
}



//! usb_get_descriptor.
//!
//! This function manages the GET DESCRIPTOR request. The device descriptor,
//! the configuration descriptor and the device qualifier are supported. All
//! other descriptors must be supported by the usb_user_get_descriptor
//! function.
//! Only 1 configuration is supported.
void usb_get_descriptor(U8 bmRequestType)
{
U16  wLength;
volatile U8  dummy;
U8  nb_byte;
U8   code *pbuffer;
U8   data_to_transfer;
U8 zlp          = FALSE;                  /* no zero length packet */
U8  string_indx     = Usb_read_byte();        /* read LSB of wValue    */
U8  descriptor_type = Usb_read_byte();        /* read MSB of wValue    */

   if (0x80 == bmRequestType)
   {
       switch (descriptor_type)
       {
        case DEVICE_DESCRIPTOR:
          data_to_transfer =  sizeof(S_usb_device_descriptor); 
          pbuffer          = (U8 code *)desc_dev_active; //&desc_dev_debug.bLength;
          dprintf("Sending device descr\r");
          break;
        case CONFIGURATION_DESCRIPTOR:
          data_to_transfer=((S_usb_configuration_descriptor code *)desc_conf_active)->wTotalLength;
          pbuffer          = (U8 code *) desc_conf_active;
          dprintf("Sending config descr\r");
          break;
        case STRING_DESCRIPTOR:
            data_to_transfer = desc_str_active[string_indx][0];
            pbuffer = (U8 __flash *) &(desc_str_active[string_indx][0]);
            break;
#ifndef LEAN_USB
       case DEVICE_QUALIFIER_DESCRIPTOR:
            goto usb_get_descriptor_unsupported;
#endif            
        default:
            if (! usb_user_get_descriptor(bmRequestType, descriptor_type, &pbuffer, &data_to_transfer))
            {
                dprintf("Unknown descriptor requst FAIL: type %x req type: %x\r",descriptor_type,bmRequestType);
                goto usb_get_descriptor_unsupported;
            }
       }
   }
   else if (! usb_user_get_descriptor(bmRequestType, descriptor_type, &pbuffer, &data_to_transfer))
   {
        dprintf("Unknown descriptor requst FAIL: type %x req type: %x\r",descriptor_type,bmRequestType);
        goto usb_get_descriptor_unsupported;
   }


   dummy=Usb_read_byte();                     //!< don't care of wIndex field
   dummy=Usb_read_byte();
   LSB(wLength) = Usb_read_byte();              //!< read wLength
   MSB(wLength) = Usb_read_byte();
   Usb_ack_receive_setup() ;                  //!< clear the receive setup flag

   if (wLength > data_to_transfer)
   {
      if ((data_to_transfer % EP_CONTROL_LENGTH) == 0) { zlp = TRUE; }
      else { zlp = FALSE; }                   //!< no need of zero length packet
   }
   else
   {
      data_to_transfer = (U8)wLength;         //!< send only requested number of data
   }

   U8 chrsent=0; 

   while((data_to_transfer != 0) && (!Is_usb_receive_out()))
   {
      while(!Is_usb_read_control_enabled());

      nb_byte=0;
      while(data_to_transfer != 0)        //!< Send data until necessary
      {
         if(nb_byte++==EP_CONTROL_LENGTH) //!< Check endpoint 0 size
         {
            break;
         }
         Usb_write_byte(*pbuffer++);
         data_to_transfer --;
         
         if (descriptor_type == STRING_DESCRIPTOR) 
         {
             if(chrsent>=4)
             {
                 if(nb_byte++==EP_CONTROL_LENGTH) //!< Check endpoint 0 size
                 {
                    Usb_send_control_in();
                    while(!Is_usb_read_control_enabled());
                    nb_byte=0;
                 }                     
                 Usb_write_byte(0x00);
                 data_to_transfer --;
             }
             else chrsent++;
         }
         
      }
      Usb_send_control_in();
   }


   if(Is_usb_receive_out()) { Usb_ack_receive_out(); return; } //!< abort from Host
   if(zlp == TRUE)
   {
     while(!Is_usb_read_control_enabled());
     Usb_send_control_in();
   }


   while(!Is_usb_receive_out());
   Usb_ack_receive_out();
   return;
   
usb_get_descriptor_unsupported:
    Usb_enable_stall_handshake();
    Usb_ack_receive_setup();
}


//! usb_set_configuration.
//!
//! This function manages the SET CONFIGURATION request. If the selected
//! configuration is valid, this function call the usb_user_endpoint_init()
//! function that will configure the endpoints following the configuration
//! number.
void usb_set_configuration( void )
{
   U8 configuration_number;

   configuration_number = Usb_read_byte();

   if (configuration_number <= NB_CONFIGURATION)
   {
      Usb_ack_receive_setup();
      usb_configuration_nb = configuration_number;
   }
   else
   {
      //!< keep that order (set StallRq/clear RxSetup) or a
      //!< OUT request following the SETUP may be acknowledged
      Usb_enable_stall_handshake();
      Usb_ack_receive_setup();
      asm("break");
      return;
   }

   Usb_send_control_in();                    //!< send a ZLP for STATUS phase

   usb_user_endpoint_init(usb_configuration_nb);  //!< endpoint configuration
   dprintf("Set configuration %x OK\r",usb_configuration_nb);

}



void usb_set_address(void)
{
   U8 dummy = Usb_read_byte();
   Usb_configure_address(dummy);

   Usb_ack_receive_setup();

   Usb_send_control_in();                    //!< send a ZLP for STATUS phase
   while(!Is_usb_in_ready());                //!< waits for status phase done
                                             //!< before using the new address
   Usb_enable_address();
}

//! usb_get_configuration.
//!
//! This function manages the GET CONFIGURATION request. The current
//! configuration number is returned.
void usb_get_configuration(void)
{
   Usb_ack_receive_setup();

   Usb_write_byte(usb_configuration_nb);
   Usb_ack_in_ready();

   while( !Is_usb_receive_out() );
   Usb_ack_receive_out();
   dprintf("GetConfiguration OK\r");

}
#ifndef LEAN_USB
//! usb_get_status.
//!
//! This function manages the GET STATUS request. The device, interface or
//! endpoint status is returned.
void usb_get_status(U8 bmRequestType)
{
U8 wIndex;
volatile U8 dummy;

   dummy=Usb_read_byte();                 //!< dummy read
   dummy=Usb_read_byte();                 //!< dummy read
   wIndex = Usb_read_byte();

   switch(bmRequestType)
   {
    case REQUEST_DEVICE_STATUS:    
    case REQUEST_INTERFACE_STATUS: Usb_ack_receive_setup();
                                   Usb_write_byte(0);
                                   break;

    case REQUEST_ENDPOINT_STATUS:  Usb_ack_receive_setup();
                                   wIndex = wIndex & MSK_EP_DIR;
                                   Usb_write_byte(endpoint_status[wIndex]);
                                   dprintf("interface status %x returned\r",endpoint_status[wIndex]);
                                   break;
    default:
                                   dprintf("unknown status request %x\r",bmRequestType);
                                   Usb_enable_stall_handshake();
                                   Usb_ack_receive_setup();
                                   return;
   }

   Usb_write_byte(0x00);
   Usb_send_control_in();

   while( !Is_usb_receive_out() );
   Usb_ack_receive_out();
}

//! usb_set_feature.
//!
//! This function manages the SET FEATURE request. The USB test modes are
//! supported by this function.
void usb_set_feature(U8 bmRequestType)
{
U8 wValue;
U8 wIndex;
volatile U8 dummy;

   if (bmRequestType == INTERFACE_TYPE)
   {
      //!< keep that order (set StallRq/clear RxSetup) or a
      //!< OUT request following the SETUP may be acknowledged
      Usb_enable_stall_handshake();
      Usb_ack_receive_setup();
      dprintf("STALLING set INTERFACE type feature\r");
      return;
   }
   else if (bmRequestType == ENDPOINT_TYPE)
   {
      wValue = Usb_read_byte();
      dummy=Usb_read_byte();                //!< dummy read

      if (wValue == FEATURE_ENDPOINT_HALT)
      {
         wIndex = (Usb_read_byte() & MSK_EP_DIR);

         if (wIndex == EP_CONTROL)
         {
            dprintf("STALLING set EP_HALT on CTRL EP\r");
            Usb_enable_stall_handshake();
            Usb_ack_receive_setup();
            return;
         }

         Usb_select_endpoint(wIndex);
         if(Is_usb_endpoint_enabled())
         {
            dprintf("STALLING set setfeature ep halt\r");
 
            Usb_enable_stall_handshake();
            Usb_select_endpoint(EP_CONTROL);
            endpoint_status[wIndex] = 0x01;
            Usb_ack_receive_setup();
            Usb_send_control_in();
         }
         else
         {
            dprintf("STALLING set setfeature ep halt not enabled anyway!\r");

            Usb_select_endpoint(EP_CONTROL);
            Usb_enable_stall_handshake();
            Usb_ack_receive_setup();
            return;
         }
      }
      else
      {
         dprintf("STALLING unknown set feature\r");
        asm("break");
         Usb_enable_stall_handshake();
         Usb_ack_receive_setup();
         return;
      }
   }
}

//! usb_clear_feature.
//!
//! This function manages the SET FEATURE request.
void usb_clear_feature(U8 bmRequestType)
{
U8 wValue;
U8 wIndex;
volatile U8 dummy;

      wValue = Usb_read_byte();
      dummy=Usb_read_byte();                //!< dummy read

      if (wValue == FEATURE_ENDPOINT_HALT)
      {
         wIndex = (Usb_read_byte() & MSK_EP_DIR);

         Usb_select_endpoint(wIndex);
         if(Is_usb_endpoint_enabled())
         {
            Usb_select_endpoint(EP_CONTROL);
            endpoint_status[wIndex] = 0x00;
            Usb_ack_receive_setup();
            Usb_send_control_in();
         }
         else
         {
            Usb_select_endpoint(EP_CONTROL);
            goto usb_clear_feature_error;
         }
      }
      else
      {
          goto usb_clear_feature_error;
      }

    return;
usb_clear_feature_error:
         Usb_enable_stall_handshake();
         Usb_ack_receive_setup();
         return;
    
}
#endif

//! usb_get_interface.
//!
//! TThis function manages the GET_INTERFACE request.
void usb_get_interface (void)
{
  Usb_enable_stall_handshake();
  Usb_ack_receive_setup();
}

//! usb_set_interface.
//!
//! TThis function manages the SET_INTERFACE request.
void usb_set_interface (void)
{
  Usb_ack_receive_setup();
  Usb_send_control_in();                    //!< send a ZLP for STATUS phase
  while(!Is_usb_in_ready());
}

#endif
